/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
 * @brief Watchdog example
 *
 * This example demonstrates the use of the watchdog. LED1 is toggled at startup.
 * Watchdog is set to 3 seconds and will cause a reset after that time. The watchdog
 * can be reloaded by pressing BUTTON0. LED0 will blink to indicate the button press.
 * Note that the watchdog does not work in debug mode. Since the chip will start up in
 * debug mode after flashing, the watchdog will only work after a powercycle.
 *
 * @image html example_board_setup_a.png "Use board setup A for this example."
 */

#ifndef BOARD_NRF6310
#define BOARD_NRF6310
#endif

#include <stdbool.h>
#include "nrf.h"
#include "nrf_gpio.h"
#include "nrf_delay.h"
#include "boards.h"

#define DELAY 200000

/**
 * Configures pin 0 for input and pin 8 for output and
 * configures GPIOTE to give interrupt on pin change.
 */
static void gpio_init(void)
{
	//Configure LEDs and button
  nrf_gpio_cfg_input(BUTTON0, NRF_GPIO_PIN_NOPULL);
	nrf_gpio_cfg_output(LED0);
	nrf_gpio_cfg_output(LED1);

	//Configure GPIOTE channel to toggle pin
  NRF_GPIOTE->CONFIG[0] =  (GPIOTE_CONFIG_POLARITY_Toggle << GPIOTE_CONFIG_POLARITY_Pos)
              | (0 << GPIOTE_CONFIG_PSEL_Pos)
              | (GPIOTE_CONFIG_MODE_Event << GPIOTE_CONFIG_MODE_Pos);	
	
  // Enable GPIOTE interrupt:
  NVIC_EnableIRQ(GPIOTE_IRQn);
  NRF_GPIOTE->INTENSET = GPIOTE_INTENSET_IN0_Set << GPIOTE_INTENSET_IN0_Pos;	
}

/** GPIOTE interrupt handler.
 * Triggered on pin 0 change
 */
void GPIOTE_IRQHandler(void)
{
  // Event causing the interrupt must be cleared
  if ((NRF_GPIOTE->EVENTS_IN[0] == 1) && (NRF_GPIOTE->INTENSET & GPIOTE_INTENSET_IN0_Msk))
  {
    NRF_GPIOTE->EVENTS_IN[0] = 0;
  }
  nrf_gpio_pin_toggle(LED0);
	NRF_WDT->RR[0] = 0x6E524635;  //Reload watchdog register 0
}

void wdt_init(void)
{
	NRF_WDT->CONFIG = (WDT_CONFIG_HALT_Pause << WDT_CONFIG_HALT_Pos) | ( WDT_CONFIG_SLEEP_Run << WDT_CONFIG_SLEEP_Pos);
	NRF_WDT->CRV = 3*32768;   //ca 3 sek. timout
	NRF_WDT->RREN |= WDT_RREN_RR0_Msk;  //Enable reload register 0
	NRF_WDT->TASKS_START = 1;
}

/**
 * main function
 * \return 0. int return type required by ANSI/ISO standard.
 */
int main(void)
{
  gpio_init();
	
	//BLINK LED 1 slowly during startup
	nrf_gpio_pin_toggle(LED1);
	nrf_delay_us(DELAY);
	nrf_gpio_pin_toggle(LED1);
	nrf_delay_us(DELAY);
	nrf_gpio_pin_toggle(LED1);
	nrf_delay_us(DELAY);
	nrf_gpio_pin_toggle(LED1);
	nrf_delay_us(DELAY);
	nrf_gpio_pin_toggle(LED1);
	nrf_delay_us(DELAY);
	nrf_gpio_pin_toggle(LED1);
	nrf_delay_us(DELAY);
	nrf_gpio_pin_toggle(LED1);
	nrf_delay_us(DELAY);

	wdt_init();
	
  while (true)
  {
		//Blink LED 1 fast until watchdog triggers reset
		nrf_gpio_pin_toggle(LED1);
		nrf_delay_us(DELAY/3);
  }
}

/**
 *@}
 **/
